
# In production this has to be set, to the name of the egg of your application:
reahlsystem.root_egg = 'hellodockernginx'   

# If using PostgreSQL:
reahlsystem.connection_uri = 'postgresql://hellodockernginx:hellodockernginx@database/hellodockernginx'

# If using MySQL:
#reahlsystem.connection_uri = 'mysql://hellodockernginx:hellodockernginx@database/hellodockernginx'

reahlsystem.debug = False