
from __future__ import print_function, unicode_literals, absolute_import, division
from reahl.doc.examples.features.dynamiccontent.dynamiccontent import DynamicContentUI

web.site_root = DynamicContentUI

