from reahl.doc.examples.howtos.customisingerrorpages.customisingerrorpages import BreakingUIWithBuiltInErrorPage, BreakingUIWithCustomErrorPage

web.site_root = BreakingUIWithBuiltInErrorPage
#web.site_root = BreakingUIWithCustomErrorPage
